using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Xml;
using System.IO;
using System.ServiceModel.Configuration;
using System.Configuration;
using System.Reflection;

namespace Microsoft.MSE
{
    public static class BindingHelper
    {
        public static Binding GetStandardBindingFromUri(Uri uri)
        {
            Binding bn = null;
            if (null == uri)
                throw new ArgumentNullException("uri");
            switch (uri.Scheme)
            {
                case "http":
                    bn = new WSHttpBinding();
                    //((WSHttpBinding)bn).Security.Mode = SecurityMode.None;
                    ((WSHttpBinding)bn).MaxReceivedMessageSize = 5000000;
                    break;
                case "net.tcp":
                    bn = new NetTcpBinding();
                    //((NetTcpBinding)bn).Security.Mode = SecurityMode.None;
                    ((NetTcpBinding)bn).MaxReceivedMessageSize = 5000000;
                    break;
                case "net.pipe":
                    bn = new NetNamedPipeBinding();
                    //((NetNamedPipeBinding)bn).Security.Mode = NetNamedPipeSecurityMode.None;
                    ((NetNamedPipeBinding)bn).MaxReceivedMessageSize = 5000000;
                    break;
            }
            return bn;
        }

        public static Binding DeserializeBinding(XmlElement BindingXml)
        {
            Binding newBinding = null;
            BindingsSection bnSec = null;

            if (null == BindingXml)
                throw new ArgumentNullException("Binding Xml");

            //make a copy since this function needs to modify the binding xml in order to deserialize it
            XmlElement bindingXmlCopy = BindingXml.Clone() as XmlElement;

            //BOTTO: get custom extensions, and remove their namespaces, 
            //add the extensions to wcf configuration and use it to load the BindingsSection
            Dictionary<string, string> extensionTypeNames = new Dictionary<string, string>();
            ExtractBindingExtensionsFromBindingConfig(bindingXmlCopy as XmlNode, extensionTypeNames);

            //BOTTO: modified to always create Configuration instance.  
            //Supports federated bindings reference other bindings (i.e. issuer binding)
            Configuration configSystem = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
            if (extensionTypeNames != null && extensionTypeNames.Count > 0)
            {
                RegisterBindingExtensions(extensionTypeNames, configSystem);
            }
                bnSec = BindingsSection.GetSection(configSystem);


            bindingXmlCopy.RemoveAllAttributes();
            bindingXmlCopy.RemoveAttribute("xmlns", "");
            bindingXmlCopy.RemoveAttribute("xmlns", MseConstants.MSENamespace);

            if (string.IsNullOrEmpty(bindingXmlCopy.OuterXml))
                throw new ArgumentException("Binding Xml - OuterXml");

            string wcfBindingXml = string.Empty;
            try
            {
                wcfBindingXml = bindingXmlCopy.OuterXml;
                if (!string.IsNullOrEmpty(bindingXmlCopy.NamespaceURI))
                    wcfBindingXml = string.Format(@"<bindings>
                                                            <{0}>{1}</{0}>
                                                        </bindings>",
                                         bindingXmlCopy.FirstChild.Name,
                                         bindingXmlCopy.FirstChild.InnerXml);

                wcfBindingXml = wcfBindingXml.Replace("xmlns=\"" + MseConstants.MSENamespace + "\"", "");


                //Deserialize Bindings Section
                //BOTTO: raw xml is required to be formatted or config subsystem will fail on some items in issuedTokenParameters
                bnSec.SectionInformation.SetRawXml(FormatXmlString(wcfBindingXml));

                List<BindingCollectionElement> bindingCollectionElements = bnSec.BindingCollections;

                BindingCollectionElement bindingCollectionElement = null;
                foreach (BindingCollectionElement bcelem in bnSec.BindingCollections)
                {
                    if (0 < bcelem.ConfiguredBindings.Count)
                    {
                        bindingCollectionElement = bcelem;
                        break;
                    }
                }

                if (null == bindingCollectionElement)
                    throw new ArgumentException("The deserialized section doesn't contain a configured binding");


                newBinding = GetBindingFromBindingCollectionElement(bindingCollectionElement, bnSec);

            }
            catch (Exception ex)
            {
                TraceHelper.TraceError("Error deserializing binding " + BindingXml.OuterXml +
                    " exception: " + ex.Message);
            }
            return newBinding;
        }

        private static void ExtractBindingExtensionsFromBindingConfig(XmlNode bindingXml, Dictionary<string, string> extensions)
        {
            for (int i = 0; i < bindingXml.ChildNodes.Count; i++)
            {
                XmlNode newNode = null;
                XmlNode node = bindingXml.ChildNodes[i];
                if (!String.IsNullOrEmpty(node.NamespaceURI))
                {
                    string nsName = node.NamespaceURI.Substring(node.Name.LastIndexOf(':') + 1);
                    if (node.NamespaceURI.StartsWith("clr-namespace:"))
                    {
                        string[] ns = node.NamespaceURI.Split(new char[] { ':', ';', '=' });
                        extensions.Add(node.LocalName, string.Format("{0}.{1},{2}", ns[1], node.LocalName, ns[3]));

                        //replace element with one w/o clr namespace
                        newNode = bindingXml.OwnerDocument.CreateElement(node.LocalName);
                        node.ParentNode.ReplaceChild(newNode, node);
                        newNode.InnerXml = node.InnerXml;
                        foreach (XmlAttribute att in node.Attributes)
                        {
                            if (!att.Value.StartsWith("clr-namespace:"))
                            {
                                XmlAttribute xatt = bindingXml.OwnerDocument.CreateAttribute(att.Name);
                                xatt.Value = att.Value;
                                newNode.Attributes.Append(xatt);
                            }
                        }
                    }
                    else
                    {
                        newNode = node;
                    }

                    if (newNode.HasChildNodes)
                        ExtractBindingExtensionsFromBindingConfig(newNode, extensions);
                }
                else
                {
                    if (node.HasChildNodes)
                        ExtractBindingExtensionsFromBindingConfig(node, extensions);
                }
            }
        }

        private static void RegisterBindingExtensions(Dictionary<string, string> extensionList, Configuration configSystem)
        {
            if (extensionList != null && extensionList.Count > 0)
            {
                ServiceModelSectionGroup configSection = ServiceModelSectionGroup.GetSectionGroup(configSystem);

                //if we don't use the supplied configSystem, this is an alternative way to prepare the extensions collection for accepting new items.
                //ExtensionsSection extensions = ConfigurationManager.GetSection("system.serviceModel/extensions") as ExtensionsSection;
                //Type ceCollectionType = typeof(ConfigurationElementCollection);
                //FieldInfo fieldInfo = ceCollectionType.GetField("bReadOnly",
                //                        BindingFlags.Instance | BindingFlags.NonPublic);
                //fieldInfo.SetValue(extensions.BindingElementExtensions, false);
                //fieldInfo.SetValue(extensions.BindingExtensions, false);

                foreach (string key in extensionList.Keys)
                {
                    Type t = Type.GetType(extensionList[key]);
                    if (t == null)
                    {
                        throw new ArgumentException(string.Format("The binding contains an unrecognized extension {0}.", key));
                    }
                    if (t.IsSubclassOf(typeof(BindingElementExtensionElement)))
                    {
                        if (!configSection.Extensions.BindingElementExtensions.ContainsKey(key))
                            configSection.Extensions.BindingElementExtensions.Add(new ExtensionElement(key, extensionList[key]));
                    }
                    else if (t.IsSubclassOf(typeof(BindingCollectionElement)))
                    {
                        if (!configSection.Extensions.BindingExtensions.ContainsKey(key))
                            configSection.Extensions.BindingExtensions.Add(new ExtensionElement(key, extensionList[key]));
                    }
                    else
                    {
                        throw new ArgumentException(string.Format("The binding contains an unrecognized extension {0}.", extensionList[key]));
                    }
                }
            }
        }

        private static Binding GetBindingFromBindingCollectionElement(
                BindingCollectionElement bindingCollectionElement,
                BindingsSection bSec)
        {
            Binding newBinding = null;
            try
            {
                if (null != bindingCollectionElement)
                {
                    newBinding = (System.ServiceModel.Channels.Binding)
                        bindingCollectionElement.BindingType.Assembly.CreateInstance(bindingCollectionElement.BindingType.FullName);

                    //BOTTO: since we are only returning 1 binding, configure it based on the first one found in the section
                    IBindingConfigurationElement bEleEx = bindingCollectionElement.ConfiguredBindings[0];
                    bEleEx.ApplyConfiguration(newBinding);
                }

            }
            catch (Exception ex)
            {
                TraceHelper.TraceError("Error in GetBindingFromBindingCollectionElement. Exception: " + ex.Message);
            }
            return newBinding;
        }
        private static string FormatXmlString(string xml)
        {
            if (string.IsNullOrEmpty(xml))
                return string.Empty;

            StringBuilder sb = new StringBuilder();
            using (StringWriter sw = new StringWriter(sb))
            {
                using (XmlTextWriter xw = new XmlTextWriter(sw))
                {
                    XmlDocument xd = new XmlDocument();
                    xd.LoadXml(xml);
                    xw.Formatting = Formatting.Indented;
                    xd.WriteTo(xw);
                    xw.Flush();
                    xw.Close();
                }
                sw.Flush();
                sw.Close();
            }
            return sb.ToString();
        }
    }
}

